@@ -4,11 +4,49 @@
 # Creation: 25 Apr 2012
 ###############################################################################
 
+#' @include utils.R
 #' @include namespace.R
-#' @include unitTests.R
-#' @include logging.R
 NULL
 
+# taken from AnnotationDbi
+make.name.tree <- function (x, recursive, what.names) 
+{
+  if (!is.character(what.names) || length(what.names) != 1) 
+    stop("'what.names' must be a single string")
+  what.names <- match.arg(what.names, c("inherited", "full"))
+  .make.name.tree.rec <- function(x, parent_name, depth) {
+    if (length(x) == 0) 
+      return(character(0))
+    x_names <- names(x)
+    if (is.null(x_names)) 
+      x_names <- rep.int(parent_name, length(x))
+    else if (what.names == "full") 
+      x_names <- paste0(parent_name, x_names)
+    else x_names[x_names == ""] <- parent_name
+    if (!is.list(x) || (!recursive && depth >= 1L)) 
+      return(x_names)
+    if (what.names == "full") 
+      x_names <- paste0(x_names, ".")
+    lapply(seq_len(length(x)), function(i) .make.name.tree.rec(x[[i]], 
+              x_names[i], depth + 1L))
+  }
+  .make.name.tree.rec(x, "", 0L)
+}
+
+.unlist2 <- function (x, recursive = TRUE, use.names = TRUE, what.names = "inherited") 
+{
+  ans <- unlist(x, recursive, FALSE)
+  if (!use.names) 
+    return(ans)
+  if (!is.character(what.names) || length(what.names) != 1) 
+    stop("'what.names' must be a single string")
+  what.names <- match.arg(what.names, c("inherited", "full"))
+  names(ans) <- unlist(make.name.tree(x, recursive, what.names), 
+      recursive, FALSE)
+  ans
+}
+unlist2 <- ns_get('AnnotationDbi::unlist2') %||% .unlist2
+
 set_libPaths <- function(lib.loc=NULL){
   ol <- Sys.getenv('R_LIBS')
   olib <- .libPaths()
@@ -33,7 +71,11 @@ set_libPaths <- function(lib.loc=NULL){
 
 #' Executing R Commands
 #' 
-#' \code{R.exec} executes a single R command via \code{\link{system2}}.
+#' Functions to execute R commands.
+#' @name R.exec
+NULL
+
+#' @describeIn R.exec executes a single R command via \code{\link{system2}}.
 #' 
 #' @param ... extra arguments that are concatenated and appended to 
 #' the command. 
@@ -52,22 +94,20 @@ R.exec <- function(..., lib.loc=NULL){
 	system(cmd, intern=interactive())
 }
 
-#' \code{R.CMD} executes R CMD commands.
+#' @describeIn R.exec executes R CMD commands.
 #' 
 #' @param cmd command to run, e.g. \sQuote{check} or \sQuote{INSTALL}.
 #' 
 #' @export
-#' @rdname R.exec
 R.CMD <- function(cmd, ...){
 	R.exec('CMD ', cmd, ' ', ...)
 }
 
-#' \code{R.SHLIB} executes R CMD SHLIB commands.
+#' @describeIn R.exec executes R CMD SHLIB commands.
 #' 
 #' @param libname name of the output compiled library
 #' 
 #' @export
-#' @rdname R.exec
 R.SHLIB <- function(libname, ...){
 	R.CMD('SHLIB', '-o ', libname, .Platform$dynlib.ext, ...)
 }
@@ -83,7 +123,6 @@ R.SHLIB <- function(libname, ...){
 compile_src <- function(pkg=NULL, load=TRUE){
 	
 	if( !is.null(pkg) ){
-		library(devtools)
 		p <- as.package(pkg)
 		path <- p$path
 	}else{
@@ -143,8 +182,8 @@ packageEnv <- function(pkg, skip=FALSE, verbose=FALSE){
 		# - as.environment('package:*') will return the correct environment
 		# in dev mode.
 		env <- if( is.environment(pkg) ) topenv(pkg)
+        else if( isLoadingNamespace(pkg) ) getLoadingNamespace(env=TRUE)
 		else if( !is.null(path.package(pkg, quiet=TRUE)) ) asNamespace(pkg)
-		else if( isLoadingNamespace(pkg) ) getLoadingNamespace(env=TRUE)
 		else if( isNamespaceLoaded(pkg) ) asNamespace(pkg)
 		else if( pkg %in% search() ) as.environment(pkg)
 		else as.environment(str_c('package:', pkg)) # dev mode
@@ -192,10 +231,10 @@ packageEnv <- function(pkg, skip=FALSE, verbose=FALSE){
 	return(e)
 }
 
-#' \code{topns_name} returns the name of the runtime sequence of top namespace(s), 
+#' @describeIn devutils returns the name of the runtime sequence of top namespace(s), 
 #' i.e. the name of the top calling package(s), from top to bottom.
 #' 
-#' \code{topns_name}: the top namespace is is not necessarily the namespace where \code{topns_name} 
+#' The top namespace is is not necessarily the namespace where \code{topns_name} 
 #' is effectively called.
 #' This is useful for packages that define functions that need to access the 
 #' calling namespace, even from calls nested into calls to another function from
@@ -207,8 +246,7 @@ packageEnv <- function(pkg, skip=FALSE, verbose=FALSE){
 #' be considered as a valid namespace.
 #' @param unique logical that indicates if the result should be reduced
 #' to contain only one occurence of each namespace. 
-#'  
-#' @rdname devutils
+#'   
 #' @export
 topns_name <- function(n=1L, strict=TRUE, unique=TRUE){
 	
@@ -244,7 +282,7 @@ topns_name <- function(n=1L, strict=TRUE, unique=TRUE){
 	if( length(res) || n>1L ) res else ''
 }
 
-#' \code{topns} returns the runtime top namespace, i.e. the namespace of 
+#' @describeIn devutils returns the runtime top namespace, i.e. the namespace of 
 #' the top calling package, possibly skipping the namespace where \code{topns} 
 #' is effectively called.
 #' This is useful for packages that define functions that need to access the 
@@ -252,7 +290,6 @@ topns_name <- function(n=1L, strict=TRUE, unique=TRUE){
 #' the same package -- in which case \code{topenv} would not give the desired 
 #' environment.
 #'  
-#' @rdname devutils
 #' @export
 topns <- function(strict=TRUE){
 	ns <- topns_name(n=1L, strict=strict)
@@ -261,7 +298,7 @@ topns <- function(strict=TRUE){
 	#packageEnv(skip=TRUE, verbose=verbose)
 }
 
-#' \code{packageName} returns the current package's name.
+#' @describeIn devutils returns the current package's name.
 #' It was made internal from version 0.16, since the package \pkg{utils} 
 #' exported its own \code{\link[utils]{packageName}} function in R-3.0.0. 
 #' 
@@ -273,7 +310,6 @@ topns <- function(strict=TRUE){
 #' @param rm.prefix logical that indicates if an eventual prefix 'package:' 
 #' should be removed from the returned string.
 #' 
-#' @rdname devutils
 #' @return a character string
 packageName <- function(envir=packageEnv(), .Global=FALSE, rm.prefix=TRUE){
 	
@@ -309,9 +345,8 @@ packageName <- function(envir=packageEnv(), .Global=FALSE, rm.prefix=TRUE){
 	}
 }
 
-#' \code{str_ns} formats a package environment/namespace for log/info messages.
+#' @describeIn devutils formats a package environment/namespace for log/info messages.
 #' 
-#' @rdname devutils
 #' @export
 str_ns <- function(envir=packageEnv()){
 	if( !is.environment(envir) )
@@ -321,18 +356,20 @@ str_ns <- function(envir=packageEnv()){
 }
 
 
-#' \code{packagePath} returns the current package's root directory, which is 
+#' @describeIn devutils returns the current package's root directory, which is 
 #' its installation/loading directory in the case of an installed package, or
 #' its source directory served by devtools. 
 #' 
 #' @param package optional name of an installed package 
 #' @param lib.loc path to a library of R packages where to search the package
 #' @param ... arguments passed to \code{\link{file.path}}.
+#' @param check logical that indicates if an error should be thrown if the path to the 
+#' package root directory cannot be found. 
+#' If this is the case and `check = FALSE`, then the function returns `NULL`. 
 #' 
-#' @rdname devutils
 #' @return a character string
 #' @export
-packagePath <- function(..., package=NULL, lib.loc=NULL){
+packagePath <- function(..., package=NULL, lib.loc=NULL, check = TRUE){
 	
 	# try to find the path from the package's environment (namespace)
 	pname <- packageName(package)
@@ -353,7 +390,12 @@ packagePath <- function(..., package=NULL, lib.loc=NULL){
 			path <- info$path
 		}
 	}
-	stopifnot( !is.null(path) && path != '' )
+  
+  # check if the path was found
+  if( !length(path) || !nzchar(path) ){
+    if( check ) stop("Could not find path to package ", package)
+    return(NULL)
+  }
 	
 	# for development packages: add inst prefix if necessary
 	if( isDevNamespace(pname) ){
@@ -370,9 +412,8 @@ packagePath <- function(..., package=NULL, lib.loc=NULL){
 	file.path(path, ...)	
 }
 
-#' \code{isPackageInstalled} checks if a package is installed.
+#' @describeIn devutils checks if a package is installed.
 #' 
-#' @rdname devutils
 #' @export
 isPackageInstalled <- function(..., lib.loc=NULL){
 	
@@ -395,7 +436,7 @@ isPackageInstalled <- function(..., lib.loc=NULL){
 #	gsub("\\\\.\\{(.)\\}", "\\1", x)
 #}
 
-#' \code{as.package} is enhanced version of \code{\link[devtools]{as.package}}, 
+#' @describeIn devutils an enhanced version of \code{\link[devtools]{as.package}}, 
 #' that is not exported not to mask the original function.
 #' It could eventually be incorporated into \code{devtools} itself.
 #' Extra arguments in \code{...} are passed to \code{\link{find.package}}. 
@@ -408,9 +449,11 @@ isPackageInstalled <- function(..., lib.loc=NULL){
 #' source files should be extracted.
 #' In this case there will be no valid path. 
 #' 
-#' @rdname devutils
-as.package <- function(x, ..., quiet=FALSE, extract=FALSE){
+#' @export
+as_package <- function(x, ..., quiet=FALSE, extract=FALSE){
 	
+	if( !requireNamespace('devtools', quietly = TRUE) ) 
+        stop("Package 'devtools' is required to load development packages")
 	
 	if( is.null(x) ) return( devtools::as.package() )
 	if( devtools::is.package(x) ) return(x)
@@ -449,6 +492,9 @@ as.package <- function(x, ..., quiet=FALSE, extract=FALSE){
 	NULL
 }
 
+# internal overload of as.package
+as.package <- as_package
+
 NotImplemented <- function(msg){
 	stop("Not implemented - ", msg)
 }
@@ -462,30 +508,60 @@ NotImplemented <- function(msg){
 #' @param list character vector containing the names of the data to load.
 #' @inheritParams utils::data
 #' @param ... other arguments eventually passed to \code{\link[utils]{data}}.
+#' @param options list of R options to set before calling \code{\link[utils]{data}}.
+#' This may be useful the data is shipped as an R script.
+#' @param stringsAsFactors logical that indicates if character columns of tabular data should be 
+#' converted into factors.
 #' 
 #' @return the loaded data.
 #' 
+#' @importFrom withr with_options
 #' @export
 #' @examples 
 #' 
 #' \dontrun{ mydata <- packageData('mydata') }
 #' 
-packageData <- function(list, envir = .GlobalEnv, ...){
+packageData <- function(list, envir = .GlobalEnv, ..., options = NULL, stringsAsFactors = getOption('stringsAsFactors', TRUE)){
 	
-	# same as utils::data if no 'list' argument
-	if( missing(list) ) return( data(..., envir=envir) )
-	# load into environment
-	data(list=list, ..., envir = envir)
-	# return the loaded data
-	if( length(list) == 1L ) get(list, envir=envir)
-	else sapply(list, get, envir=envir, simplify=FALSE)
+  # workaround for when this function is called under batchtools
+  # See https://github.com/mllg/batchtools/issues/195
+  if( !length(options) ){
+    options <- setNames(list(1), tempfile())
+    
+  }
+  
+	withr::with_options(options, {
+		# same as utils::data if no 'list' argument
+		if( missing(list) ) return( data(..., envir=envir) )
+		# load into environment
+		data(list=list, ..., envir = envir)
+		# return the loaded data
+		.get <- function(x, ...){
+			res <- get(x, ...)
+			# force factors into character vectors
+			if( !stringsAsFactors && is.data.frame(res) ){
+				for(n in colnames(res)[sapply(res, is.factor)]){
+					res[[n]] <- as.character(res[[n]])
+				}
+			}
+			res
+		}
+		if( length(list) == 1L ) .get(list, envir=envir)
+		else sapply(list, .get, envir=envir, simplify=FALSE)
+    
+	})
 	
 }
 
-#' \code{ldata} loads a package data in the parent frame.
+#' @describeIn packageData loads a package data in the parent frame.
 #' It is a shortcut for \code{packageData(list, ..., envir=parent.frame())}.
 #' 
-#' @rdname packageData
+#' @param error a logical that indicates whether an error should be thrown if 
+#' the requested data cannot be found.
+#' @param simplify logical that indicates if queries of one object only (i.e. argument `list` 
+#' is of length one) should return the data object itself. 
+#'  
+#' @importFrom assertthat assert_that is.scalar
 #' @export
 #' @examples
 #' 
@@ -496,7 +572,149 @@ packageData <- function(list, envir = .GlobalEnv, ...){
 #' }
 #' }
 #' 
-ldata <- function(list, ...){
+ldata <- function(list, ..., package = NULL, error = TRUE, simplify = TRUE){
+  # parameter check
+  if( missing(list) ) list <- NULL
+  assert_that(is.null(list) || (is.vector(list) && is.character(list))
+                , msg = "Invalid argument 'list': value must be NULL or a character vector.")
+  assert_that(is.null(package) || isNZString(package), msg = "Invalid argument 'package': value must be NULL or a non-empty string.")
+  assert_that(is.scalar(error) && is.logical(error), msg = "Invalid argument 'error': value must be a single logical.")
+  assert_that(is.scalar(simplify) && is.logical(simplify), msg = "Invalid argument 'simplify': value must be a single logical.")
+  ##
+  
+  # load data.list
+  if( is.null(package) || qrequire(package, character.only = TRUE) ){
+    dlist <- list.data(package = package)[["data"]]
+    
+  }else{
+    msg <- sprintf("Could not find data objects %s in package '%s': package not found.", str_out(list, Inf), package)
+    if( error ) stop(msg)
+    else warning(msg)
+    dlist <- character()
+    
+  }
+  # limit to requested data list
+  if( !missing(list) || length(list) ){
+    dlist <- dlist[dlist %in% list]
+    
+  }
+  # pre-check that the requested data exist
+  if( !error ){
+    # initialize result list
+    res <- sapply(list, function(x) NULL, simplify = FALSE)
+    list <- intersect(list, dlist)
+    
+  }
+  
+  # fetch the data objects
 	e <- parent.frame()
-	packageData(list=list, ..., envir=e)
+  res_data <- sapply(list, function(l) packageData(list=l, ..., envir=e, package = package), simplify = FALSE)
+  
+  # return 
+  if( error ){
+    # simplify if requested
+    if( simplify && length(res_data) == 1L ) res_data <- res_data[[1L]]
+    return(res_data)
+    
+  }else{
+    if( length(miss <- setdiff(names(res), names(res_data))) ){
+      pkg_str <- "in the currently loaded packages"
+      if( !is.null(package) ) pkg_str <- paste0("in package ", package)
+      warning(sprintf("Could not find data object(s) %s: %s", pkg_str, str_out(miss, Inf)))
+      
+    }
+    #
+    for(n in names(res_data)){
+      res[[n]] <- res_data[[n]]
+    }
+    if( simplify && length(res) == 1L ) res <- res[[1L]]
+    res
+    
+  }
+  
+}
+
+#' List Package Data Objects
+#' 
+#' Lists data objects that are shipped within package(s).
+#' 
+#' @param package a single character string that specifies the name of a particular
+#' package where to look for data objects.
+#' 
+#' @return a `data.frame` object with columns:
+#' 
+#'   * `package`: name of the package that holds the data object.
+#'   * `data`: name of the key to use in [utils::data] or [ldata] to load the data object.
+#'   * `object`: name of the (sub-)object that is contained in the data object.
+#' 
+#' @seealso [utils::data], [ldata]
+#' @export
+#' @examples
+#' 
+#' # list all data objects
+#' head(list.data())
+#' 
+#' # list all data objects in package 'datasets'
+#' subset(list.data("datasets"), data %in% "beavers")
+#' 
+list.data <- function(package = NULL){
+  dlist <- data(package = package)[["results"]]
+  m <- str_match(dlist[, "Item"], "^(([^(]+)|(([^(]+)\\(([^)]+)\\)))$")
+  key <- ifelse(!is.na(m[, 3L]), m[, 3L], m[, 6L])
+  stopifnot(!anyNA(key))
+  obj_name <- ifelse(!is.na(m[, 3L]), m[, 3L], m[, 5L])
+  stopifnot(!anyNA(obj_name))
+  data.frame(package = dlist[, "Package"], data = key, object = obj_name, stringsAsFactors = FALSE)
+  
+}
+
+
+#' Generate a Loading Script for Development Packages
+#' 
+#' Writes a script file that contains code that loads a given development package.
+#' 
+#' This is useful when we want to load a development package in `batchtools` registries:
+#' 
+#' ```
+#' library(devtools)
+#' library(batchtools)
+#' 
+#' load_all("path/to/pkgA")
+#' makeRegistry(..., source = load_all_file("pkgA"))
+#' ```
+#' 
+#' @param path a character string that contains the path to the development package.
+#' @param package the name of the package for which the loading script must be generated.
+#' It must be a package that has already been loaded with [devtools::load_all] in the current
+#' session, so that its path can be retrieved.
+#' @param dest the path to script file to create (as a character string).
+#' If not provided, then the script is written in a temporary .R file with prefix 
+#' `"load_all_<pkgname>_"`.
+#' 
+#' @return a character string that contains the path to the script file.
+#' 
+#' @export
+load_all_file <- function(path = path.package(package), package, dest = NULL){
+  
+  if( !missing(path) && !missing(package)  ){
+    stop("Arguments 'path' and 'package' are exclusive: only one of them can be provided.")
+    
+  }
+  if( !requireNamespace('devtools', quietly = TRUE) ) 
+    stop("Package 'devtools' is required to load development package information.")
+  
+  # load package object
+  if( !missing(package) ) path <- path.package(package) 
+  pkg <- devtools::as.package(path)
+   
+  # define destination script file
+  if( is.null(dest) ) dest <- tempfile(paste0("load_all_", pkg[["package"]], "_"), fileext = ".R")
+  # write file
+  cat(sprintf("# package loader script (generated on: %s) \nmessage(\"* Loader script '%s'\")\nmessage(\"* Loading development package %s from: '%s'\")\ndevtools::load_all(\"%s\")\n"
+          , date(), dest, pkg[["package"]], pkg[["path"]], pkg[["path"]])
+      , file = dest)
+  
+  # return path to script
+  dest
+  
 }
